//TrckrX MPH RPM Coolant Temp in LCD
//by Pavel Vasilev Cornell Univesity 2012
//ECE 4760 Final Project

#define DEBUG 1

/* Includes */

#include <avr/pgmspace.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <stddef.h>
#include <inttypes.h>
#include "lcd_lib.h"
#include <math.h>
#include "uart.h"

FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);

//fatfs libraries
#include "fatfs/ff.h"
#include "fatfs/ff.c"
#include "fatfs/mmc.c"
#include "fatfs/diskio.h"

/* Global/System Constants */
#define TRUE	1
#define FALSE	0

#define sbi(data,bit)			(data |= (1<<bit))	// Set Bit
#define cbi(data,bit) 			(data &= ~(1<<bit))	// Clear Bit
#define gbi(data,bit)			(data & (1<<bit))	// Get Bit
#define tbi(data,bit)			(data ^= (1<<bit))	// Toggle Bit

//State Machine States
#define Initial 1
#define Trans_EchoOff 2
#define Rec_EchoOff 3
#define Trans_Reset 4
#define Rec_Reset 5
#define Trans_VSS 6
#define Rec_VSS 7
#define Trans_MPG 8
#define Rec_MPG	9 
#define Trans_Protocol 10
#define Rec_Protocol 11
#define Trans_RPM 12
#define Rec_RPM	13
#define Trans_Coolant 14
#define Rec_Coolant	15

//Data conversion
#define airflowrate_convert(a,b)	((256*(a))+(b))*0.01;
#define speed_convert(a)			floor((a)*(0.621371192));
#define rpm_convert(a,b)			floor(((256*(a))+(b))/4);
#define temp_convert(a)				floor(((a)*(1.8))+32); 

//Macro timers
#define t1 1000 //write to SD card
#define diskInterval 10 //10ms timer
#define t2 100 //increment system time ms
#define t3 50 //debouncer
#define t4 200 //LCD timer
#define t5 50 //ADC timer

//State machine state names
#define NoPush 1 
#define MaybePush 2
#define Pushed 3
#define MaybeNoPush 4

#define countmax 3

unsigned char PushFlag;		//message indicating a button push 
unsigned char PushState;	//state machine  

//Time Keeping 
volatile unsigned int SDTimer,realTimer,buttonTimer,lcdTimer,gTimer;	

/* Global Variables */
uint8_t rx_buffer[128];
uint8_t rx_buffer_index;

uint8_t lcd_buffer[17];	// LCD display buffer
//enum {MCU_STATE_IDLE, MCU_STATE_TX};
//uint8_t error_buffer[40];

//LCD Messages
const int8_t LCD_product[] PROGMEM = "TrckrX ///M3 E36";
const int8_t LCD_name[] PROGMEM = "Pavel Vasilev";
const int8_t LCD_VSS[] PROGMEM = "MPH\0";
const int8_t LCD_MPG[] PROGMEM = "MPG\0";
const int8_t LCD_RPM[] PROGMEM = "RPM\0";
const int8_t LCD_Temp[] PROGMEM = "Temp\0";
const int8_t LCD_F[] PROGMEM = " F\0";
const int8_t LCD_empty[] PROGMEM = " \0";

//Meassages

uint8_t Reset[4] = "ATZ\r";
uint8_t EchoOff[5] = "ATE0\r";
uint8_t ATL[5] = "ATL1\r";
uint8_t VSS[5] = "010D\r";
uint8_t MAF[5] = "0110\r";
uint8_t RPM[5] = "010C\r";
uint8_t TEMP[5] = "0105\r";

/* Prototype Functions */
void init_uart0_rxtx(void); 
void uart0_send_byte_array(uint8_t *data, uint8_t length);
//void uart0_send_byte(uint8_t data);
void uart0_parse_rx(uint8_t rx_data);
void init_lcd(void); 
void initialize(void); 
void state_machine(void);  
void debounce_machine(void); 
     
/* Function that writes the A/D reading to the SD card */
void writeSDcard(void);
void openLog(void);
void closeLog(void);

//ADC for G-meter
void acc(void);

unsigned int incount;            //increasing counter

/* Function that initiates the file system */
void init_fs(void);    
//declaration for disk_timerproc()
void disk_timerproc();

//Update the current system in days/hours/minutes at a 1ms 
void IncrementTime();
void TimeStamp();

void printFRcode(FRESULT fResult);  

//State Machine Marker
int state;  

//Car data
int mph,kph,temperature,rpm;
float mpg,maf;

uint8_t filter_vss,filter_41,filter_maf,filter_rpm,filter_temp;

//unsigned int maf and rpm;
unsigned int temp_maf1, temp_maf2;
unsigned int temp_rpm1, temp_rpm2;

//Number of times the file has been written to
unsigned int counter;

unsigned char flag,wFlag;

volatile char diskTime;   

unsigned char openFlag,flagus; //is the file open?

unsigned int reading; //dummy reading		

//time
volatile unsigned int millis;
volatile unsigned int secs;
volatile unsigned int mins;
volatile unsigned int hours;
volatile unsigned int day;
volatile unsigned int month;
volatile unsigned int year;

volatile unsigned int SDsec;
volatile unsigned int SDmin; 

unsigned char disk_timer;

//file system pointers
FIL logFile;
FIL* logPtr; //= &logFile
FATFS fs;
FATFS* fsPtr;

unsigned char diskInitFlag;
unsigned char diskMountFlag;

float a,a_peak,V,V_peak,Vref,Vd,g,g_peak;
char Ain,Aprev;

unsigned int incount;            //increasing counter
unsigned char peak;                 //indicates at peak position 

int flagV;	

/******************************************************************************************/
//timer 0 overflow ISR
ISR (TIMER0_COMPA_vect)
{
	//Decrement the time if not already zero
	if (buttonTimer>0) --buttonTimer;
    if (SDTimer>0)    --SDTimer; 			    
	if (realTimer>0) --realTimer;
	if (realTimer == 0) {
			realTimer = t2; 
			IncrementTime();
	}
	if (diskTime>0) --diskTime;
    if (diskTime == 0) {
            diskTime = diskInterval;
            disk_timerproc();
    }
	if (lcdTimer>0)	--lcdTimer;
	if (gTimer>0)	--gTimer;
}

/******************************************************************************************/
/**
 * UART transmission function for byte arrays
 *
 * Can be used to transmitt strings
 * Length terminated (versus null terminated)
 */
void uart0_send_byte_array(uint8_t *data, uint8_t length) {
	for (uint8_t i=0; i<length; i++) {
		while ( !( UCSR0A & (1<<UDRE0)) );	// Wait for empty buffer
		UDR0 = *data++;						// Put data into buffer, sends the data 
	}
}

void updateLCD (void) 
{
	if(mph <10){
		LCDGotoXY(0, 1);
		sprintf(lcd_buffer, "   %d", mph);
		LCDstring(lcd_buffer, strlen(lcd_buffer));
		}
	if ((mph >= 10) && (mph < 100)){
		LCDGotoXY(0, 1);
		sprintf(lcd_buffer, "  %d", mph);
		LCDstring(lcd_buffer, strlen(lcd_buffer));
		}
	if (mph >= 100) {
		LCDGotoXY(0, 1);
		sprintf(lcd_buffer, " %d", mph);
		LCDstring(lcd_buffer, strlen(lcd_buffer));
		}
	if((rpm<10)&&(rpm==0)){
		LCDGotoXY(5, 1);
		sprintf(lcd_buffer, "   %d", rpm);
		LCDstring(lcd_buffer, strlen(lcd_buffer));
		}

	else if(rpm<100){
		LCDGotoXY(5, 1);
		sprintf(lcd_buffer, "  %d", rpm);
		LCDstring(lcd_buffer, strlen(lcd_buffer));
		CopyStringtoLCD(LCD_empty, 9, 1);
		}

	else if(rpm<1000){
		LCDGotoXY(5, 1);
		sprintf(lcd_buffer, " %d", rpm);
		LCDstring(lcd_buffer, strlen(lcd_buffer));
		CopyStringtoLCD(LCD_empty, 9, 1);
		}
	else if(rpm>=1000){
		LCDGotoXY(5, 1);
		sprintf(lcd_buffer, "%d", rpm);
		LCDstring(lcd_buffer, strlen(lcd_buffer));
		}
	
	if (temperature <= 10)
		{
		LCDGotoXY(11, 1);
		sprintf(lcd_buffer,"%d",temperature);
		LCDstring(lcd_buffer, strlen(lcd_buffer));
		CopyStringtoLCD(LCD_F, 14, 1);
		}
	else if (temperature<0 && temperature>10)
		{
		LCDGotoXY(11, 1);
		sprintf(lcd_buffer," %d", temperature);
		LCDstring(lcd_buffer, strlen(lcd_buffer));
		CopyStringtoLCD(LCD_F, 14, 1);
		}
	else if((temperature >= 0) && (temperature <10))
		{
		LCDGotoXY(11, 1);
		sprintf(lcd_buffer, "  %d", (temperature));
		LCDstring(lcd_buffer, strlen(lcd_buffer));
		CopyStringtoLCD(LCD_F, 14, 1);
		}
	else if ((temperature >= 10) && (temperature < 100))
		{
		LCDGotoXY(11, 1);
		sprintf(lcd_buffer, " %d", temperature);
		LCDstring(lcd_buffer, strlen(lcd_buffer));
		CopyStringtoLCD(LCD_F, 14, 1);	
		}
	else if (temperature >= 100)
		{
		LCDGotoXY(11, 1);
		sprintf(lcd_buffer, "%d", temperature);
		LCDstring(lcd_buffer, strlen(lcd_buffer));
		CopyStringtoLCD(LCD_F, 14, 1);
		}
}

void acc(void)
{ 
	ADMUX = (1 << ADLAR) | 0b01000000;
	ADCSRA = (1<<ADEN) | (1<<ADSC) + 7 ;
	//_delay_ms(100);
	Vref = 4.95;
		
	while ((ADCSRA & (1 << ADSC)));
		
	Ain = ADCH;
	a=Ain;
	
	V = (a/255)*Vref;

	//calibration calculation	
	if (flagV == 0){
		Vd = 2.5-V;
		flagV = 1;
	}
	
	V=V+Vd;

	g = ((V/2.5)-1)*1.5;

	
	if (Ain>Aprev) //increasing
 	{
    	if (incount<countmax) incount++;
		//Aprev = Ain;
  	}
  
  	if (Ain<Aprev) //decreasing
  	{
     if (incount==countmax) 
     {
           peak=1;
		   a_peak = Aprev;
		   //a = Ain;
		   V_peak = (a_peak/255)*Vref;
		   V_peak = V + Vd;
		   g_peak = ((V_peak/2.5)-1)*1.5;
           incount=0;
     }
     else if (incount<countmax & incount>0) 
     { 
        incount--;
     }
  	}
	
	Aprev = Ain;

	ADCSRA |= (1<<ADSC) ; //starts next conveRsion
}

//Parsing function based on current state
void uart0_parse_rx(uint8_t rx_data) {

if (state == Rec_EchoOff){
		if (rx_data == 0x3E) {
			rx_buffer_index = 0;
			state = Trans_Reset;
		}
		else if (rx_data >= 0x20 && rx_data <= 0x5F) {
			rx_buffer[rx_buffer_index++] = rx_data;
		}
	}
	else if (state == Rec_Protocol){
		if (rx_data == 0x3E) {
			rx_buffer_index = 0;
			state = Trans_RPM;
		}
		else if (rx_data >= 0x20 && rx_data <= 0x5F) {
			rx_buffer[rx_buffer_index++] = rx_data;
		}
	}
	else if (state == Rec_Reset){
		if (rx_data == 0x3E) {
			rx_buffer_index = 0;
			state = Trans_VSS;
		}
		else if (rx_data >= 0x20 && rx_data <= 0x5F) {
			rx_buffer[rx_buffer_index++] = rx_data;
		}
	}
	else if (state == Rec_VSS){
		
			if (rx_data == 0x3E){
				sscanf(rx_buffer,"%X %X",&filter_41,&filter_vss);

				if((filter_41 == 0x41)&&(filter_vss == 0x0D)){
					sscanf(rx_buffer,"%*s %*s %X",&kph);
					mph = speed_convert(kph);
				}	
				rx_buffer_index = 0;	
				state = Trans_RPM;
			}
			else if (rx_data >= 0x20 && rx_data <= 0x5F) {
				rx_buffer[rx_buffer_index++] = rx_data;
			}
			
	}
	else if (state == Rec_RPM){
			if (rx_data == 0x3E){
				sscanf(rx_buffer,"%X %X",&filter_41,&filter_rpm);
				//fprintf(stdout,"%X %X\n\r", filter_41, filter_vss);

				if((filter_41 == 0x41)&&(filter_rpm == 0x0C)){
					sscanf(rx_buffer,"%*s %*s %X %X",&temp_rpm1,&temp_rpm2);
					rpm = rpm_convert(temp_rpm1,temp_rpm2);
				}	
				rx_buffer_index = 0;	
				state = Trans_Coolant;
			}
			else if (rx_data >= 0x20 && rx_data <= 0x5F) {
				rx_buffer[rx_buffer_index++] = rx_data;
			}
	}

	else if(state == Rec_Coolant){
		if (rx_data == 0x3E){
				_delay_ms(30); //was 50
				sscanf(rx_buffer,"%X %X",&filter_41,&filter_temp);

				if((filter_41 == 0x41)&&(filter_temp == 0x05)){
					sscanf(rx_buffer,"%*s %*s %X",&temperature);
					temperature=temp_convert(temperature - 40);
				}	
				rx_buffer_index = 0;	
				state = Trans_VSS;
			}
			else if (rx_data >= 0x20 && rx_data <= 0x5F) {
				rx_buffer[rx_buffer_index++] = rx_data;
			}
	}

}

/******************************************************************************************/
/**
 * Main program
 */
int main(void) {
	initialize();
	init_fs(); //init disk and mount filesystem
	// Main program loop
	while(1) {
		state_machine();
		if (buttonTimer==0) {buttonTimer = t3;	debouncer();}
	 	if ((PushFlag)&&(flag==0)){
				PushFlag = 0;
				flag = 1; 
				PORTD = 0b11110100;
				openLog();
				wFlag = 1; //flag for continous SD writing;
		}
		if (SDTimer==0){
			SDTimer=t1;
			if (wFlag) {TimeStamp(1); writeSDcard();}
			}
		if((PushFlag)&&(flag==1)){
				PushFlag = 0;
				flag = 0;
				wFlag = 0;
				PORTD = 0b11111100;		
				closeLog();
				TimeStamp(0);
		}	
		if (lcdTimer==0){lcdTimer=t4; updateLCD();}
		if (gTimer==0){gTimer=t5; acc();}

	}
}
/******************************************************************************************/
void initialize(void){

	//set up timer 0 for 1 mSec ticks
 	TIMSK0 = 2;		//turn on timer 0 cmp match ISR 
  	OCR0A = 249;  	//set the compare reg to 250 time ticks
  	TCCR0A = 0b00000010; // turn on clear-on-match
  	TCCR0B = 0b00000011;	// clock prescalar to 64	

	rx_buffer_index = 0;

	DDRD = 0b00001010;	// PORT D is an input except for TX and LED
	PORTD =0b11111100;	// PORT D pullup MAKE SURE THATS OK FOR UART
	
	init_uart0_rxtx();		// Initialize UART TX
	init_lcd();

	state = Initial;
	
	kph = 0;
	mph = 0;

	mpg = 0;

	maf = 0;
	temp_maf1 = 0;
	temp_maf2 = 0;

	rpm = 0;
	temp_rpm1 = 0;
	temp_rpm2 = 0;

	temperature = 0;
	
	PushFlag = 0;
	PushState = NoPush;
	
	//init the task timer
	SDTimer=t1;
	realTimer = t2;
	buttonTimer = t3; 
	lcdTimer=t4; 
	gTimer=t5;
	
	//init time
	millis = 0;
	secs = 0;
	mins = 0;
	hours = 0;
	day = 22; //dummy day
	month = 4; //dummy month
	year = 20; //dummy year
	
	//init flags
    openFlag = 0;
    diskInitFlag = 0;
    diskMountFlag = 0;

    counter = 0;
	
	//init the UART -- uart_init() is in uart.c
	uart_init();
	stdout = stdin = stderr = &uart_str;
	fprintf(stdout,"Serial Debug\n\r");
	
	flagV = 0;
	Aprev = 0;
	ADMUX = (1 << ADLAR) | 0b01000010;
	ADCSRA = (1<<ADEN) | (1<<ADSC) + 7 ; 
	_delay_ms(100);
	 
	sei();
}
/******************************************************************************************/
void state_machine(void){

	switch(state){
	case Initial:

		CopyStringtoLCD(LCD_product, 0, 0);		
		CopyStringtoLCD(LCD_name, 0, 1);		

		_delay_ms(2000);

		LCDclr();
		state = Trans_EchoOff;
			
	break;

	case Trans_EchoOff:
		uart0_send_byte_array(EchoOff,5);
		_delay_ms(500);
		state = Rec_EchoOff;
	break;

	case Rec_EchoOff:
		if(UCSR0A & (1<<RXC0)) {	
			uart0_parse_rx(UDR0);
		}
	break;
	
	case Trans_Protocol:
		uart0_send_byte_array(ATL,5);
		_delay_ms(500);
		//UBRR0L = 25;
		state = Rec_Protocol;
	break;

	case Rec_Protocol:
		if(UCSR0A & (1<<RXC0)) {	
			uart0_parse_rx(UDR0);
		}
	break;
	
	case Trans_Reset:
		uart0_send_byte_array(Reset,4);
		state = Rec_Reset;

	break;

	case Rec_Reset:
		if(UCSR0A & (1<<RXC0)) {
			uart0_parse_rx(UDR0);
		}

	break;
	
	case Trans_VSS:		
		CopyStringtoLCD(LCD_VSS, 1, 0);
		uart0_send_byte_array(VSS,5);
		state = Rec_VSS;
	break;

	case Rec_VSS:
		if(UCSR0A & (1<<RXC0)) {
			uart0_parse_rx(UDR0);
		}
	break;
	
	case Trans_MPG:
		uart0_send_byte_array(MAF,5);
		state = Rec_MPG;
	break;
	
	case Rec_MPG:
		if(UCSR0A & (1<<RXC0)) {
			uart0_parse_rx(UDR0);
		}
	break;

	case Trans_RPM:
		CopyStringtoLCD(LCD_RPM, 6, 0);
		uart0_send_byte_array(RPM,5);
		state = Rec_RPM;
	break;
	
	case Rec_RPM:
		if(UCSR0A & (1<<RXC0)) {
			uart0_parse_rx(UDR0);
		}
	break;

	case Trans_Coolant:
		CopyStringtoLCD(LCD_Temp, 11, 0);
		uart0_send_byte_array(TEMP,5);
		state = Rec_Coolant;
	break;
	
	case Rec_Coolant:
		if(UCSR0A & (1<<RXC0)) {
			uart0_parse_rx(UDR0);
		}
	break;

	}
}

void init_lcd(void) {
	LCDinit();	//initialize the display
	LCDcursorOFF();
	LCDclr();				//clear the display	
}

/* Inits filesystem on the SD card */
void init_fs() {
	DSTATUS dstatus;
    
    //PORTD &= ~(1 << PORTD2); // LED on
    
	dstatus = disk_initialize(0);

    #ifdef DEBUG
	fprintf(stdout, "Disk Initiliazation: %d\n\r", dstatus);
	if(dstatus == STA_NOINIT) fprintf(stdout, "STA_NOINIT ");
	else if(dstatus == STA_NODISK) fprintf(stdout, "STA_NODISK ");
	else if(dstatus == STA_PROTECT) fprintf(stdout, "STA_PROTECT ");
    #endif
	
    if(dstatus == 0) diskInitFlag = 1; //Disk has been Initialized
	
	FRESULT result;
	//init pointer
	fsPtr = &fs;
	logPtr = &logFile;
	//set up file system
	result = f_mount(0, fsPtr);
    #ifdef DEBUG
	fprintf(stdout, "Initiating filesystem mount: %d\n\r", result);
    #endif
    
    if(result == 0) diskMountFlag = 1; //Disk has been mounted
}

void IncrementTime()
{
	millis++;	
	if(millis == 10) {
		secs++;
		millis = 0;
		if(secs == 60) {
			mins++;
			secs = 0;
			if(mins == 60) {
				hours++;
				mins = 0;
				if(hours == 24) { hours = 0; }
			}
		}
	}
}

void TimeStamp(int reset)
{
	if(reset == 0){
		SDsec = 0;
		SDmin = 0;
	}
	if(reset == 1){
		SDsec++;
		if(SDsec == 60) {
			SDmin++;
			SDsec = 0;
		}
	}
}	

DWORD get_fattime()
{
	return      ((DWORD)(2012 - 1980) << 25)
            | ((DWORD)5 << 21)
            | ((DWORD)(day+1) << 16)
            | ((DWORD)hours << 11)
            | ((DWORD)mins << 5)
            | ((DWORD)(millis >> 10) >> 1);
    // div 1024 is close enough to div 1000 since the time is
    // pretty made up in this case anyway   
}

//Opens the file to write to
void openLog(void) {
	FRESULT fileResult;
	char filePath[16];
	unsigned int fileCounter = 0;
	
	if(diskInitFlag && diskMountFlag) {
	//Try to create a new log file. It will fail if the log exists, keep incrementing log file counter until we succeed
	do {
		snprintf(filePath, 16, "log%03d.csv", fileCounter);
		fileResult = f_open(logPtr, filePath,  FA_WRITE | FA_CREATE_NEW); //FA_WRITE | FA_CREATE_NEW
	} while (fileResult == FR_EXIST && ++fileCounter < 1000);
	if (fileResult) {
		#ifdef DEBUG
		fprintf(stdout, "Log creation FAILED: %d\r\n", fileResult);
		printFRcode(fileResult);
		#endif
	}
	else {
		#ifdef DEBUG
		fprintf(stdout, "Log Created: %s \r\n", filePath);
		#endif
		openFlag = 1;
	}
	}
    else {
        #ifdef DEBUG
        fprintf(stdout, "Disk not mounted/initialized can't open file\n\r");
        #endif
    }
}

//Closes the file
void closeLog(void) {
    //only try to close if the file has been opened
	FRESULT fileResult;
	if(openFlag) {
        fileResult = f_close(&logFile);
        if(fileResult) {
            #ifdef DEBUG
            fprintf(stdout, "File Closing Failed: ");
            printFRcode(fileResult);
            #endif
        }
        else {
            #ifdef DEBUG
            fprintf(stdout, "File Close.\n\r");
            #endif
            openFlag = 0;
        }
    }
}


void writeSDcard(void) {
    FRESULT fileResult;
    char str[30];
    unsigned int length;
    unsigned int bytesWrote;
	
	uint8_t buff[10];
    //dtostrf(mpg, 3, 1, buff);
	dtostrf(g_peak, 1, 3, buff);
    //Write string to file
    snprintf(str, 30, "%02d:%02d,%d,%d,%d,%s\n\r", SDmin,SDsec,mph,rpm,temperature,buff);// mins, secs,
    
	//zero them out
	g_peak = 0;
	dtostrf(g, 1, 3, buff);
	
	#ifdef DEBUG
    fprintf(stdout, str); //for debugging
	#endif
	
	length = strlen(str);
	fileResult = f_write(logPtr, str, strlen(str), &bytesWrote);
	
	#ifdef DEBUG
	if (fileResult | (bytesWrote != length)) {
		fprintf(stdout, "Log write error. Result %d, Tried %d, Wrote %d\r\n", fileResult, length, bytesWrote);
	}
	else {
		fprintf(stdout, "Log write succeeded.\r\n");
	}
	#endif
	//sync the filesystem (flush buffers) so it will power down cleanly
	f_sync(logPtr);
}

//*******************************   
//Button Debouncer  
void debouncer(void) 
{
  switch (PushState)
  {
     case NoPush: 
        if (~PIND & 0x04) PushState=MaybePush; //0x04 = 0b00000100 0b11111011
        else PushState=NoPush;
        break;
     case MaybePush:
        if (~PIND & 0x04) 
        {
           PushState=Pushed;   
           PushFlag=1;
        }
        else PushState=NoPush;
        break;
     case Pushed:  
        if (~PIND & 0x04) PushState=Pushed; 
        else PushState=MaybeNoPush;    
        break;
     case MaybeNoPush:
        if (~PIND & 0x04) PushState=Pushed; 
        else 
        {
           PushState=NoPush;
           PushFlag=0;
        }    
        break;
  }
}

//Print out file result code
void printFRcode(FRESULT fResult) {
	switch(fResult) {
		case FR_OK:
			fprintf(stdout, "FR OK\n\r");
			break;
		case FR_DISK_ERR:
			fprintf(stdout, "FR Disk Error\n\r");
			break;
		case FR_INT_ERR:
			fprintf(stdout, "FR_INT_ERR\n\r");
			break;
		case FR_NOT_READY:
			fprintf(stdout, "FR_NOT_READY\n\r");
			break;
		case FR_NO_FILE:
			fprintf(stdout, "FR_NO_FILE\n\r");
			break;			
		case FR_NO_PATH:
			fprintf(stdout, "FR_NO_PATH\n\r");
			break;			
		case FR_INVALID_NAME:
			fprintf(stdout, "FR_INVALID_NAME\n\r");
			break;	
		case FR_DENIED:
			fprintf(stdout, "FR_DENIED\n\r");
			break;			
		case FR_EXIST:
			fprintf(stdout, "FR_EXIST\n\r");
			break;			
		case FR_INVALID_OBJECT:
			fprintf(stdout, "FR_INVALID_OBJECT\n\r");
			break;	
		case FR_WRITE_PROTECTED:
			fprintf(stdout, "FR_WRITE_PROTECTED\n\r");
			break;	
		case FR_INVALID_DRIVE:
			fprintf(stdout, "FR_INVALID_DRIVE\n\r");
			break;	
		case FR_NOT_ENABLED:
			fprintf(stdout, "FR_NOT_ENABLED\n\r");
			break;		
		case FR_NO_FILESYSTEM:	
			fprintf(stdout, "FR_NO_FILESYSTEM\n\r");
			break;
		case FR_MKFS_ABORTED:	
			fprintf(stdout, "FR_MKFS_ABORTED\n\r");
			break;
		case FR_TIMEOUT:
			fprintf(stdout, "FR_TIMEOUT\n\r");
			break;			
		default:
			fprintf(stdout, "Unknown FR code\n\r");
			break;
	}
}

/******************************************************************************************/
/**
 * Initialize UART RX
 */
void init_uart0_rxtx(void){
	UBRR0L = 103;		// 9600 Baud Rate for 16 MHz Clock

	UCSR0A = 0b00000000;// 7 - UART RX complete flag
						// 6 - UART TX complete flag
						// 5 - UART DRE flag
						// 4 - Frame error flag
						// 3 - Data overrun flag
						// 2 - Parity error flag
						// 1 - Double UART speed mode
						// 0 - Multi processor communication mode

	UCSR0B = 0b00011000; // 7 - RX Complete Interrupt Enable
						// 6 - TX Complete Interrupt Enable
						// 5 - UART DRE Interrupt Enable
						// 4 - Enable receiver
						// 3 - Enable transmitter
						// 2 - Character size
						// 1:0 - Ninth data bit

	UCSR0C = 0b00000110;// 7:6 - UART Mode Select
						//		00 - Asynchronous UART
						//		01 - Synchronous UART
						//		10 - Reserved
						//		11 - Master SPI
						// 5:4 - Parity mode
						//		00 - Disabled
						//		01 - Reserved
						//		10 - Enabled, Even
						//		11 - Enabled, Odd
						// 3 - Select stop bit length
						//		0 - 1-bit
						//		1 - 2-bit
						// 2:1 - Character size, with UCSRB.2
						//		000 - 5 bit
						//		001 - 6 bit
						//		010 - 7 bit
						//		011 - 8 bit
						//		100 - Reserved
						//		101 - Reserved
						//		110 - Reserved
						//		111 - 9 bit
						// 0 - Select clock polarity
						//		0 - Rising XCK edge
						//		1 - Falling XCK edge 						
}
